/**
 * Helper function for parsing plain route configurations
 * based on react-router createRoutes handler.
 *
 * currently supported keys:
 * - path
 * - component
 * - childRoutes
 * - indexRoute
 *
 * Usage example:
 * const routes = createRoutes([
 *  {
 *    path        : '/',
 *    component   : App,
 *    indexRoute  : {
 *      component     : Home,
 *    },
 *    childRoutes : [
 *      {
 *        path : 'films/',
 *        component : Films,
 *        childRoutes : {
 *          path : 'detail/:id',
 *          component : FilmDetail,
 *        }
 *      },
 *      {
 *        path : '/*',
 *        component : NoMatch
 *      }
 *    ]
 *  }
 * ]);
 *
 * Usage on Router JSX
 * <Router history={browserHistory} children={routes} />
 */
import { VNode } from 'inferno';
import Component from 'inferno-component';
import { IRouteHook } from './Route';
export interface IPlainRouteConfig {
    path: string;
    component: Component<any, any>;
    indexRoute?: IPlainRouteConfig;
    childRoutes?: IPlainRouteConfig | IPlainRouteConfig[];
    children?: VNode | VNode[];
    onEnter?: IRouteHook;
    onLeave?: IRouteHook;
}
declare const _default: (routeConfig: IPlainRouteConfig[]) => VNode[];
export default _default;
